@@ -21,34 +21,27 @@
 #  endif
 #endif
 
-#define TICKS_PER_DAY       INT64_C(86400000000)
-#define TICKS_PER_HOUR      INT64_C(3600000000)
-#define TICKS_PER_MIN       INT64_C(60000000)
-#define TICKS_PER_SEC       INT64_C(1000000)
-#define TICKS_PER_MSEC      INT64_C(1000)
-#define TICKS_PER_USEC      INT64_C(1)
+#define SECS_PER_DAY        86400
+#define SECS_PER_HOUR       3600
+#define SECS_PER_MIN        60
 
 #define UNIX_EPOCH          INT64_C(62135683200)  /* 1970-01-01T00:00:00 */
 
 #define MIN_EPOCH_SEC       INT64_C(-62135596800) /* 0001-01-01T00:00:00 */
 #define MAX_EPOCH_SEC       INT64_C(253402300799) /* 9999-12-31T23:59:59 */
-    
-#define MIN_OFFSET          -1080
-#define MAX_OFFSET          1080
 
 #define VALID_EPOCH_SEC(s) \
     (s >= MIN_EPOCH_SEC && s <= MAX_EPOCH_SEC)
 
-#define VALID_OFFSET(o) \
-    (o >= MIN_OFFSET && o <= MAX_OFFSET)
-
 typedef struct {
-    int64_t ticks;
+    int64_t sec;
+    int32_t nsec;
     int32_t offset;
 } moment_t;
 
 moment_t    THX_moment_from_epoch(pTHX_ int64_t sec, IV usec, IV offset);
 moment_t    THX_moment_with_offset(pTHX_ const moment_t *mt, IV offset);
+moment_t    THX_moment_with_nanosecond(pTHX_ const moment_t *mt, IV nsec);
 
 int64_t     moment_epoch(const moment_t *mt);
 
@@ -75,14 +68,18 @@ int         moment_minute(const moment_t *mt);
 int         moment_second(const moment_t *mt);
 int         moment_millisecond(const moment_t *mt);
 int         moment_microsecond(const moment_t *mt);
+int         moment_nanosecond(const moment_t *mt);
 
 int         moment_offset(const moment_t *mt);
 
-#define moment_from_epoch(sec, usec, offset) \
-    THX_moment_from_epoch(aTHX_ sec, usec, offset)
+#define moment_from_epoch(sec, nsec, offset) \
+    THX_moment_from_epoch(aTHX_ sec, nsec, offset)
 
 #define moment_with_offset(self, offset) \
     THX_moment_with_offset(aTHX_ self, offset)
 
+#define moment_with_nanosecond(self, nsec) \
+    THX_moment_with_nanosecond(aTHX_ self, nsec)
+
 #endif
 
